@@ -7,7 +7,7 @@ use tower_lsp::lsp_types::DiagnosticSeverity;
 use tree_sitter::Point;
 
 use crate::CMakeNodeKinds;
-use crate::config::{self, CMAKE_LINT_CONFIG};
+use crate::config::{self, CONFIG};
 use crate::consts::TREESITTER_CMAKE_LANGUAGE;
 use crate::utils::{include_is_module, remove_quotation_and_replace_placeholders};
 
@@ -63,7 +63,7 @@ pub fn checkerror<P: AsRef<Path>>(
         for item in v.inner {
             error_info.inner.push(item);
         }
-    };
+    }
 
     result
 }
@@ -83,7 +83,7 @@ fn run_cmake_lint<P: AsRef<Path>>(
         return run_extra_lint(path);
     }
     let mut info = vec![];
-    let max_len = CMAKE_LINT_CONFIG.line_max_words;
+    let max_len = CONFIG.line_max_words;
     for (index, line) in contexts.iter().enumerate() {
         let len = line.len();
         if len > max_len {
@@ -95,7 +95,7 @@ fn run_cmake_lint<P: AsRef<Path>>(
                 row: index,
                 column: 0,
             };
-            let message = format!("[C0301] Line too long ({}/{})", len, max_len);
+            let message = format!("[C0301] Line too long ({len}/{max_len})");
             info.push(ErrorInformation {
                 start_point,
                 end_point,
@@ -188,11 +188,15 @@ fn checkerror_inner<P: AsRef<Path>>(
         let x = ids.start_position().column;
         let y = ids.end_position().column;
         let name = &newsource[h][x..y];
-        if use_lint && !config::CMAKE_LINT.lint_match(name.chars().all(|a| a.is_uppercase())) {
+        if use_lint
+            && let Some(hint) = config::CONFIG
+                .command_case
+                .and_then(|lint| lint.check(name))
+        {
             output.push(ErrorInformation {
                 start_point: ids.start_position(),
                 end_point: ids.end_position(),
-                message: config::CMAKE_LINT.hint.clone(),
+                message: hint.to_owned(),
                 severity: Some(DiagnosticSeverity::HINT),
             });
         }
@@ -462,7 +466,7 @@ fn gammer_passed_check_1() {
     let source = include_str!("../assets_for_test/gammar/include_check.cmake");
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(&source, None).unwrap();
+    let thetree = parse.parse(source, None).unwrap();
 
     let input = thetree.root_node();
     assert_eq!(
@@ -488,7 +492,7 @@ fn gammer_passed_check_2() {
     let source = include_str!("../assets_for_test/gammar/pass_test.cmake");
     let mut parse = tree_sitter::Parser::new();
     parse.set_language(&TREESITTER_CMAKE_LANGUAGE).unwrap();
-    let thetree = parse.parse(&source, None).unwrap();
+    let thetree = parse.parse(source, None).unwrap();
 
     assert!(
         checkerror_inner(
@@ -511,21 +515,15 @@ aa.cmake:56: [C0301] Line too long (143/80)
 aa.cmake:57: [C0301] Line too long (145/80)"#;
     let re = regex::Regex::new(RE_MATCH_LINT_RESULT).unwrap();
     for s in input.split('\n') {
-        match re.captures(s) {
-            Some(m) => {
-                assert!(m.name("line").is_some() && m.name("message").is_some());
-                let row = m.name("line").unwrap().as_str().parse().unwrap_or(1) - 1;
-                let column = if let Some(m) = m.name("column") {
-                    m.as_str().parse().unwrap()
-                } else {
-                    0
-                };
-                let message = m.name("message").unwrap().as_str().to_owned();
-                println!("{row}:{column} -- {message}");
-            }
-            None => {
-                assert!(false);
-            }
-        }
+        let m = re.captures(s).unwrap();
+        assert!(m.name("line").is_some() && m.name("message").is_some());
+        let row = m.name("line").unwrap().as_str().parse().unwrap_or(1) - 1;
+        let column = if let Some(m) = m.name("column") {
+            m.as_str().parse().unwrap()
+        } else {
+            0
+        };
+        let message = m.name("message").unwrap().as_str().to_owned();
+        println!("{row}:{column} -- {message}");
     }
 }
